@@ -42,7 +42,7 @@
 #include "ogr_p.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: ogr_srs_usgs.cpp 971ad299681ca1ea2e1b800e88209f426b77e9aa 2018-04-17 12:14:43 +0200 Even Rouault $")
+CPL_CVSID("$Id: ogr_srs_usgs.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
 
 /************************************************************************/
 /*  GCTP projection codes.                                              */
@@ -209,7 +209,7 @@ static double OGRSpatialReferenceUSGSUnpackRadian( double dfVal )
  * @param padfPrjParams Array of 15 coordinate system parameters. These
  * parameters differs for different projections.
  *
- *        <h4>Projection Transformation Package Projection Parameters</h4>
+ * <h4>Projection Transformation Package Projection Parameters</h4>
  * <pre>
  * ----------------------------------------------------------------------------
  *                         |                    Array Element
@@ -387,7 +387,7 @@ static double OGRSpatialReferenceUSGSUnpackRadian( double dfVal )
  * If a datum code is zero or greater, the semimajor and semiminor axis are
  * defined by the datum code as found in the following table:
  *
- *      <h4>Supported Datums</h4>
+ * <h4>Supported Datums</h4>
  * <pre>
  *       0: Clarke 1866 (default)
  *       1: Clarke 1880
@@ -401,7 +401,7 @@ static double OGRSpatialReferenceUSGSUnpackRadian( double dfVal )
  *       9: Airy
  *      10: Modified Everest
  *      11: Modified Airy
- *      12: Walbeck
+ *      12: WGS 84
  *      13: Southeast Asia
  *      14: Australian National
  *      15: Krassovsky
@@ -759,8 +759,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
             else
             {
                 CPLError( CE_Warning, CPLE_AppDefined,
-                          "Failed to lookup datum code %d, likely due to "
-                          "missing GDAL gcs.csv file.  "
+                          "Failed to lookup datum code %d. "
                           "Falling back to use WGS84.",
                           static_cast<int>(iDatum) );
                 SetWellKnownGeogCS("WGS84");
@@ -784,8 +783,6 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
     if( IsLocal() || IsProjected() )
         SetLinearUnits( SRS_UL_METER, 1.0 );
 
-    FixupOrdering();
-
     return OGRERR_NONE;
 }
 
