@@ -90,7 +90,7 @@ CADBuffer::CADBuffer(size_t size) : m_nBitOffsetFromStart(0)
     m_pBuffer = new char[size];
     // zero memory
     memset(m_pBuffer, 0, size);
-    m_guard = m_pBuffer + size;
+    m_nSize = size;
 }
 
 CADBuffer::~CADBuffer()
@@ -110,15 +110,14 @@ unsigned char CADBuffer::Read2B()
     size_t        nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t        nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * p2BByte = m_pBuffer + nByteOffset;
-    if(p2BByte + 2 > m_guard)
+    if(nByteOffset + 2 > m_nSize)
     {
         m_bEOB = true;
         return 0;
     }
 
     unsigned char a2BBytes[2];
-    memcpy( a2BBytes, p2BByte, 2 );
+    memcpy( a2BBytes, m_pBuffer + nByteOffset, 2 );
 
     switch( nBitOffsetInByte )
     {
@@ -143,15 +142,14 @@ unsigned char CADBuffer::Read3B()
     size_t        nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t        nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * p3BByte = m_pBuffer + nByteOffset;
-    if(p3BByte + 2 > m_guard)
+    if(nByteOffset + 2 > m_nSize)
     {
         m_bEOB = true;
         return 0;
     }
 
     unsigned char a3BBytes[2];
-    memcpy( a3BBytes, p3BByte, 2 );
+    memcpy( a3BBytes, m_pBuffer + nByteOffset, 2 );
 
     switch( nBitOffsetInByte )
     {
@@ -182,15 +180,14 @@ unsigned char CADBuffer::Read4B()
     size_t        nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t        nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * p4BByte = m_pBuffer + nByteOffset;
-    if(p4BByte + 2 > m_guard)
+    if(nByteOffset + 2 > m_nSize)
     {
         m_bEOB = true;
         return 0;
     }
 
     unsigned char a4BBytes[2];
-    memcpy( a4BBytes, p4BByte, 2 );
+    memcpy( a4BBytes, m_pBuffer + nByteOffset, 2 );
 
     switch( nBitOffsetInByte )
     {
@@ -227,15 +224,14 @@ double CADBuffer::ReadBITDOUBLE()
     size_t nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * pDoubleFirstByte = m_pBuffer + nByteOffset;
-    if(pDoubleFirstByte + 9 > m_guard)
+    if(nByteOffset + 9 > m_nSize)
     {
         m_bEOB = true;
         return 0.0;
     }
 
     unsigned char aDoubleBytes[9]; // maximum bytes a single double can take.
-    memcpy( aDoubleBytes, pDoubleFirstByte, 9 );
+    memcpy( aDoubleBytes, m_pBuffer + nByteOffset, 9 );
 
     switch( BITCODE )
     {
@@ -294,6 +290,12 @@ double CADBuffer::ReadBITDOUBLE()
 void CADBuffer::SkipBITDOUBLE()
 {
     unsigned char BITCODE = Read2B();
+    size_t nByteOffset      = m_nBitOffsetFromStart / 8;
+    if(nByteOffset + 9 > m_nSize)
+    {
+        m_bEOB = true;
+        return;
+    }
 
     switch( BITCODE )
     {
@@ -314,14 +316,13 @@ short CADBuffer::ReadRAWSHORT()
     size_t nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * pShortFirstByte = m_pBuffer + nByteOffset;
-    if(pShortFirstByte + 3 > m_guard)
+    if(nByteOffset + 3 > m_nSize)
     {
         m_bEOB = true;
         return 0;
     }
     unsigned char aShortBytes[3];
-    memcpy( aShortBytes, pShortFirstByte, 3 );
+    memcpy( aShortBytes, m_pBuffer + nByteOffset, 3 );
 
     switch( nBitOffsetInByte )
     {
@@ -349,15 +350,14 @@ double CADBuffer::ReadRAWDOUBLE()
     size_t nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * pDoubleFirstByte = m_pBuffer + nByteOffset;
-    if(pDoubleFirstByte + 9 > m_guard)
+    if(nByteOffset + 9 > m_nSize)
     {
         m_bEOB = true;
         return 0.0;
     }
 
     unsigned char aDoubleBytes[9];
-    memcpy( aDoubleBytes, pDoubleFirstByte, 9 );
+    memcpy( aDoubleBytes, m_pBuffer + nByteOffset, 9 );
 
     switch( nBitOffsetInByte )
     {
@@ -397,15 +397,14 @@ int CADBuffer::ReadRAWLONG()
     size_t nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * pLongFirstByte = m_pBuffer + nByteOffset;
-    if(pLongFirstByte + 5 > m_guard)
+    if(nByteOffset + 5 > m_nSize)
     {
         m_bEOB = true;
         return 0;
     }
 
     unsigned char aLongBytes[5];
-    memcpy( aLongBytes, pLongFirstByte, 5 );
+    memcpy( aLongBytes, m_pBuffer + nByteOffset, 5 );
 
     switch( nBitOffsetInByte )
     {
@@ -437,14 +436,13 @@ bool CADBuffer::ReadBIT()
     size_t nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * pBoolByte = m_pBuffer + nByteOffset;
-    if(pBoolByte >= m_guard)
+    if(nByteOffset >= m_nSize)
     {
         m_bEOB = true;
         return false;
     }
 
-    unsigned char resultVal = ( pBoolByte[0] >> ( 7 - nBitOffsetInByte ) ) & binary( 00000001 );
+    unsigned char resultVal = ( m_pBuffer[nByteOffset] >> ( 7 - nBitOffsetInByte ) ) & binary( 00000001 );
     ++m_nBitOffsetFromStart;
 
     return resultVal == 0 ? false : true;
@@ -457,15 +455,14 @@ short CADBuffer::ReadBITSHORT()
     size_t nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * pShortFirstByte = m_pBuffer + nByteOffset;
-    if(pShortFirstByte + 4 > m_guard)
+    if(nByteOffset + 4 > m_nSize)
     {
         m_bEOB = true;
         return 0;
     }
 
     unsigned char aShortBytes[4]; // maximum bytes a single short can take.
-    memcpy( aShortBytes, pShortFirstByte, 4 );
+    memcpy( aShortBytes, m_pBuffer + nByteOffset, 4 );
 
     switch( BITCODE )
     {
@@ -516,15 +513,14 @@ unsigned char CADBuffer::ReadCHAR()
     size_t        nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t        nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * pCharFirstByte = m_pBuffer + nByteOffset;
-    if(pCharFirstByte + 2 > m_guard)
+    if(nByteOffset + 2 > m_nSize)
     {
         m_bEOB = true;
         return result;
     }
 
     unsigned char aCharBytes[2]; // maximum bytes a single char can take.
-    memcpy( aCharBytes, pCharFirstByte, 2 );
+    memcpy( aCharBytes, m_pBuffer + nByteOffset, 2 );
 
     result = ( aCharBytes[0] << nBitOffsetInByte );
     result |= ( aCharBytes[1] >> ( 8 - nBitOffsetInByte ) );
@@ -551,81 +547,37 @@ std::string CADBuffer::ReadTV()
 long CADBuffer::ReadUMCHAR()
 {
     long result = 0;
-    /*bool   negative = false;*/
     size_t nByteOffset = m_nBitOffsetFromStart / 8;
     // TODO: bit offset is calculated, but function has nothing to do with it.
     /*size_t nBitOffsetInByte = m_nBitOffsetFromStart % 8;*/
 
-    const char * pMCharFirstByte = m_pBuffer + nByteOffset;
-    if(pMCharFirstByte + 8 > m_guard)
+    if(nByteOffset + 8 > m_nSize)
     {
         m_bEOB = true;
         return 0;
     }
     unsigned char aMCharBytes[8]; // 8 bytes is maximum.
-    //memcpy( aMCharBytes, pMCharFirstByte, 8 );
+    //memcpy( aMCharBytes, m_pBuffer + nByteOffset, 8 );
 
-    size_t MCharBytesCount = 0;
-    for( size_t i = 0; i < 8; ++i )
+    unsigned char nMCharBytesCount = 0;
+    for( unsigned char i = 0; i < 8; ++i )
     {
         aMCharBytes[i] = ReadCHAR();
-        ++MCharBytesCount;
+        ++nMCharBytesCount;
         if ( !( aMCharBytes[i] & binary( 10000000 ) ) )
         {
             break;
         }
-    }
-
-    SwapEndianness( aMCharBytes, MCharBytesCount ); // LSB to MSB
-
-    for( size_t i = 0; i < MCharBytesCount; ++i )
-    {
         aMCharBytes[i] &= binary( 01111111 );
     }
 
-    // TODO: this code doesn't cover case when char.bytescount > 3, but its
-    //       possible on large files.
-    // I just can't write an algorithm that does this.
-    switch( MCharBytesCount )
+    int nOffset = 0;
+    for(unsigned char i = 0; i < nMCharBytesCount; ++i)
     {
-        case 1:
-            break;
-        case 2:
-        {
-            char tmp = aMCharBytes[0] & binary( 00000001 );
-            aMCharBytes[0] = aMCharBytes[0] >> 1;
-            aMCharBytes[1] |= ( tmp << 7 );
-            break;
-        }
-        case 3:
-        {
-            unsigned char tmp1 = aMCharBytes[0] & binary( 00000011 );
-            unsigned char tmp2 = aMCharBytes[1] & binary( 00000001 );
-            aMCharBytes[0] = aMCharBytes[0] >> 2;
-            aMCharBytes[1] = aMCharBytes[1] >> 1;
-            aMCharBytes[1] |= ( tmp1 << 6 );
-            aMCharBytes[2] |= ( tmp2 << 7 );
-            break;
-        }
-        case 4:
-        {
-            unsigned char tmp1 = aMCharBytes[0] & binary( 00000111 );
-            unsigned char tmp2 = aMCharBytes[1] & binary( 00000011 );
-            unsigned char tmp3 = aMCharBytes[2] & binary( 00000001 );
-            aMCharBytes[0] = aMCharBytes[0] >> 3;
-            aMCharBytes[1] = aMCharBytes[1] >> 2;
-            aMCharBytes[2] = aMCharBytes[2] >> 1;
-            aMCharBytes[1] |= ( tmp1 << 5 );
-            aMCharBytes[2] |= ( tmp2 << 6 );
-            aMCharBytes[3] |= ( tmp3 << 7 );
-            break;
-        }
+        unsigned long nVal = aMCharBytes[i];
+        result += nVal << nOffset;
+        nOffset += 7;
     }
-
-    SwapEndianness( aMCharBytes, MCharBytesCount ); // MSB to LSB
-
-    memcpy( &result, aMCharBytes, MCharBytesCount );
-
     return result;
 }
 
@@ -638,84 +590,43 @@ long CADBuffer::ReadMCHAR()
     // TODO: bit offset is calculated, but function has nothing to do with it.
     /*size_t nBitOffsetInByte = nBitOffsetFromStart % 8;*/
 
-    const char * pMCharFirstByte = m_pBuffer + nByteOffset;
-    if(pMCharFirstByte + 8 > m_guard)
+    if(nByteOffset + 8 > m_nSize)
     {
         m_bEOB = true;
         return 0;
     }
     unsigned char aMCharBytes[8]; // 8 bytes is maximum.
-    //memcpy( aMCharBytes, pMCharFirstByte, 8 );
 
-    size_t MCharBytesCount = 0;
-    for( size_t i = 0; i < 8; ++i )
+    unsigned char nMCharBytesCount = 0;
+    for( unsigned char i = 0; i < 8; ++i )
     {
         aMCharBytes[i] = ReadCHAR();
-        ++MCharBytesCount;
+        ++nMCharBytesCount;
         if ( !( aMCharBytes[i] & binary( 10000000 ) ) )
         {
             break;
         }
-    }
-
-    SwapEndianness( aMCharBytes, MCharBytesCount ); // LSB to MSB
-
-    if ( ( aMCharBytes[0] & binary( 01000000 ) ) == binary( 01000000 ) )
-    {
-        aMCharBytes[0] &= binary( 10111111 );
-        negative = true;
-    }
-
-    for( size_t i = 0; i < MCharBytesCount; ++i )
-    {
         aMCharBytes[i] &= binary( 01111111 );
     }
 
-    // TODO: this code doesn't cover case when char.bytescount > 3, but its
-    //       possible on large files.
-    // I just can't write an algorithm that does this.
-    switch( MCharBytesCount )
+    if ( ( aMCharBytes[nMCharBytesCount - 1] & binary( 01000000 ) ) == binary( 01000000 ) )
     {
-        case 1:
-            break;
-        case 2:
-        {
-            char tmp = aMCharBytes[0] & binary( 00000001 );
-            aMCharBytes[0] = aMCharBytes[0] >> 1;
-            aMCharBytes[1] |= ( tmp << 7 );
-            break;
-        }
-        case 3:
-        {
-            unsigned char tmp1 = aMCharBytes[0] & binary( 00000011 );
-            unsigned char tmp2 = aMCharBytes[1] & binary( 00000001 );
-            aMCharBytes[0] = aMCharBytes[0] >> 2;
-            aMCharBytes[1] = aMCharBytes[1] >> 1;
-            aMCharBytes[1] |= ( tmp1 << 6 );
-            aMCharBytes[2] |= ( tmp2 << 7 );
-            break;
-        }
-        case 4:
-        {
-            unsigned char tmp1 = aMCharBytes[0] & binary( 00000111 );
-            unsigned char tmp2 = aMCharBytes[1] & binary( 00000011 );
-            unsigned char tmp3 = aMCharBytes[2] & binary( 00000001 );
-            aMCharBytes[0] = aMCharBytes[0] >> 3;
-            aMCharBytes[1] = aMCharBytes[1] >> 2;
-            aMCharBytes[2] = aMCharBytes[2] >> 1;
-            aMCharBytes[1] |= ( tmp1 << 5 );
-            aMCharBytes[2] |= ( tmp2 << 6 );
-            aMCharBytes[3] |= ( tmp3 << 7 );
-            break;
-        }
+        aMCharBytes[nMCharBytesCount - 1] &= binary( 10111111 );
+        negative = true;
     }
 
-    SwapEndianness( aMCharBytes, MCharBytesCount ); // MSB to LSB
-
-    memcpy( &result, aMCharBytes, MCharBytesCount );
+    int nOffset = 0;
+    for(unsigned char i = 0; i < nMCharBytesCount; ++i)
+    {
+        unsigned long nVal = aMCharBytes[i];
+        result += nVal << nOffset;
+        nOffset += 7;
+    }
 
     if( negative )
+    {
         result *= -1;
+    }
 
     return result;
 }
@@ -725,8 +636,8 @@ unsigned int CADBuffer::ReadMSHORT()
     unsigned int  result = 0;
     unsigned char aMShortBytes[8]; // 8 bytes is maximum.
 
-    // TODO: this function doesnot support MSHORTS longer than 4 bytes. ODA says
-    //       its impossible, but not sure.
+    // TODO: this function does not support MSHORTS longer than 4 bytes. ODA says
+    //       it's impossible, but not sure.
     size_t MShortBytesCount = 2;
     aMShortBytes[0] = ReadCHAR();
     aMShortBytes[1] = ReadCHAR();
@@ -863,14 +774,13 @@ int CADBuffer::ReadBITLONG()
     size_t nByteOffset      = m_nBitOffsetFromStart / 8;
     size_t nBitOffsetInByte = m_nBitOffsetFromStart % 8;
 
-    const char * pLongFirstByte = m_pBuffer + nByteOffset;
-    if(pLongFirstByte + 5 > m_guard)
+    if(nByteOffset + 5 > m_nSize)
     {
         m_bEOB = true;
         return 0;
     }
     unsigned char aLongBytes[5]; // maximum bytes a single short can take.
-    memcpy( aLongBytes, pLongFirstByte, 5 );
+    memcpy( aLongBytes, m_pBuffer + nByteOffset, 5 );
 
     switch( BITCODE )
     {
@@ -930,6 +840,12 @@ void CADBuffer::SkipTV()
 void CADBuffer::SkipBITLONG()
 {
     unsigned char BITCODE = Read2B();
+    size_t nByteOffset      = m_nBitOffsetFromStart / 8;
+    if(nByteOffset + 5 > m_nSize)
+    {
+        m_bEOB = true;
+        return;
+    }
     switch( BITCODE )
     {
         case BITLONG_NORMAL:
@@ -949,6 +865,12 @@ void CADBuffer::SkipBITLONG()
 void CADBuffer::SkipBITSHORT()
 {
     unsigned char BITCODE = Read2B();
+    size_t nByteOffset      = m_nBitOffsetFromStart / 8;
+    if(nByteOffset + 4 > m_nSize)
+    {
+        m_bEOB = true;
+        return;
+    }
     switch( BITCODE )
     {
         case BITSHORT_NORMAL:
@@ -967,6 +889,12 @@ void CADBuffer::SkipBITSHORT()
 
 void CADBuffer::SkipBIT()
 {
+    size_t nByteOffset      = m_nBitOffsetFromStart / 8;
+    if(nByteOffset >= m_nSize)
+    {
+        m_bEOB = true;
+        return;
+    }
     ++m_nBitOffsetFromStart;
 }
 
@@ -999,7 +927,7 @@ void CADBuffer::Seek(size_t offset, CADBuffer::SeekPosition position)
         m_nBitOffsetFromStart += offset;
         break;
     case END:
-        m_nBitOffsetFromStart = m_guard - m_pBuffer - offset;
+        m_nBitOffsetFromStart = m_nSize - offset;
         break;
     default:
         break;
