@@ -37,7 +37,7 @@
 #include <algorithm>
 #include <memory>
 
-CPL_CVSID("$Id: ogrgeojsonseqdriver.cpp d026fc87ad02cbeea9389a1732b2ef3fc33c79ef 2019-03-23 13:45:03 +0100 Even Rouault $")
+CPL_CVSID("$Id: ogrgeojsonseqdriver.cpp 057fef6f31d50e01ce841d56c10c0a71df1bad8e 2020-01-04 19:05:08 +0100 Even Rouault $")
 
 constexpr char RS = '\x1e';
 
@@ -95,8 +95,7 @@ class OGRGeoJSONSeqLayer final: public OGRLayer
         GIntBig m_nTotalFeatures = 0;
         GIntBig m_nNextFID = 0;
 
-        void Init();
-        json_object* GetNextObject();
+        json_object* GetNextObject(bool bLooseIdentification);
 
     public:
         OGRGeoJSONSeqLayer(OGRGeoJSONSeqDataSource* poDS,
@@ -104,6 +103,8 @@ class OGRGeoJSONSeqLayer final: public OGRLayer
                            VSILFILE* fp);
         ~OGRGeoJSONSeqLayer();
 
+        bool Init(bool bLooseIdentification);
+
         void ResetReading() override;
         OGRFeature* GetNextFeature() override;
         OGRFeatureDefn* GetLayerDefn() override { return m_poFeatureDefn; }
@@ -263,8 +264,6 @@ OGRGeoJSONSeqLayer::OGRGeoJSONSeqLayer(OGRGeoJSONSeqDataSource* poDS,
     poSRSWGS84->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     m_poFeatureDefn->GetGeomFieldDefn(0)->SetSpatialRef(poSRSWGS84);
     poSRSWGS84->Release();
-
-    Init();
 }
 
 /************************************************************************/
@@ -281,7 +280,7 @@ OGRGeoJSONSeqLayer::~OGRGeoJSONSeqLayer()
 /*                               Init()                                 */
 /************************************************************************/
 
-void OGRGeoJSONSeqLayer::Init()
+bool OGRGeoJSONSeqLayer::Init(bool bLooseIdentification)
 {
     if( STARTS_WITH(m_poDS->GetDescription(), "/vsimem/") ||
         !STARTS_WITH(m_poDS->GetDescription(), "/vsi") )
@@ -294,7 +293,7 @@ void OGRGeoJSONSeqLayer::Init()
 
     while( true )
     {
-        auto poObject = GetNextObject();
+        auto poObject = GetNextObject(bLooseIdentification);
         if( !poObject )
             break;
         if( OGRGeoJSONGetType(poObject) == GeoJSONObject::eFeature )
@@ -310,6 +309,8 @@ void OGRGeoJSONSeqLayer::Init()
     m_nFileSize = 0;
     m_nIter = 0;
     m_oReader.FinalizeLayerDefn( this, m_osFIDColumn );
+
+    return m_nTotalFeatures > 0;
 }
 
 /************************************************************************/
@@ -322,10 +323,13 @@ void OGRGeoJSONSeqLayer::ResetReading()
     // Undocumented: for testing purposes only
     const size_t nBufferSize = static_cast<size_t>(std::max(1,
         atoi(CPLGetConfigOption("OGR_GEOJSONSEQ_CHUNK_SIZE", "40960"))));
-    m_osBuffer.resize(nBufferSize);
+    const size_t nBufferSizeValidated =
+        nBufferSize > static_cast<size_t>(100 * 1000 * 1000) ?
+            static_cast<size_t>(100 * 1000 * 1000) : nBufferSize;
+    m_osBuffer.resize(nBufferSizeValidated);
     m_osFeatureBuffer.clear();
-    m_nPosInBuffer = nBufferSize;
-    m_nBufferValidSize = nBufferSize;
+    m_nPosInBuffer = nBufferSizeValidated;
+    m_nBufferValidSize = nBufferSizeValidated;
     m_nNextFID = 0;
 }
 
@@ -333,7 +337,7 @@ void OGRGeoJSONSeqLayer::ResetReading()
 /*                           GetNextObject()                            */
 /************************************************************************/
 
-json_object* OGRGeoJSONSeqLayer::GetNextObject()
+json_object* OGRGeoJSONSeqLayer::GetNextObject(bool bLooseIdentification)
 {
     m_osFeatureBuffer.clear();
     while( true )
@@ -398,6 +402,12 @@ json_object* OGRGeoJSONSeqLayer::GetNextObject()
             }
         }
 
+        while( !m_osFeatureBuffer.empty() &&
+               (m_osFeatureBuffer.back() == '\r' ||
+                m_osFeatureBuffer.back() == '\n')  )
+        {
+            m_osFeatureBuffer.resize(m_osFeatureBuffer.size()-1);
+        }
         if( !m_osFeatureBuffer.empty() )
         {
             json_object* poObject = nullptr;
@@ -409,6 +419,10 @@ json_object* OGRGeoJSONSeqLayer::GetNextObject()
                 return poObject;
             }
             json_object_put(poObject);
+            if( bLooseIdentification )
+            {
+                return nullptr;
+            }
         }
     }
 }
@@ -421,7 +435,7 @@ OGRFeature* OGRGeoJSONSeqLayer::GetNextFeature()
 {
     while( true )
     {
-        auto poObject = GetNextObject();
+        auto poObject = GetNextObject(false);
         if( !poObject )
             return nullptr;
         OGRFeature* poFeature;
@@ -721,7 +735,26 @@ bool OGRGeoJSONSeqDataSource::Open( GDALOpenInfo* poOpenInfo,
         return false;
     }
     SetDescription( poOpenInfo->pszFilename );
-    m_poLayer.reset(new OGRGeoJSONSeqLayer(this, osLayerName.c_str(), fp));
+    auto poLayer = new OGRGeoJSONSeqLayer(this, osLayerName.c_str(), fp);
+    const bool bLooseIdentification =
+        nSrcType == eGeoJSONSourceService &&
+        !STARTS_WITH_CI(poOpenInfo->pszFilename, "GeoJSONSeq:");
+    if( bLooseIdentification )
+    {
+        CPLPushErrorHandler(CPLQuietErrorHandler);
+    }
+    auto ret = poLayer->Init(bLooseIdentification);
+    if( bLooseIdentification )
+    {
+        CPLPopErrorHandler();
+        CPLErrorReset();
+    }
+    if( !ret )
+    {
+        delete poLayer;
+        return false;
+    }
+    m_poLayer.reset(poLayer);
     return true;
 }
 
