@@ -44,7 +44,7 @@
 #include "ogr_core.h"
 #include "ogr_geometry.h"
 
-CPL_CVSID("$Id: gmlfeatureclass.cpp 1398568b34b36256a98108fcbdc32bae6d73d84f 2018-03-13 10:22:19Z Even Rouault $")
+CPL_CVSID("$Id: gmlfeatureclass.cpp 6c215be4458b32ab4278c9f64ed7e31099a5021c 2018-06-23 13:09:29 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                          GMLFeatureClass()                           */
@@ -100,6 +100,7 @@ void GMLFeatureClass::StealProperties()
     CPLFree(m_papoProperty);
     m_papoProperty = nullptr;
     m_oMapPropertyNameToIndex.clear();
+    m_oMapPropertySrcElementToIndex.clear();
 }
 
 /************************************************************************/
@@ -158,10 +159,9 @@ int GMLFeatureClass::GetPropertyIndexBySrcElement( const char *pszElement,
                                                    int nLen ) const
 
 {
-    for( int i = 0; i < m_nPropertyCount; i++ )
-        if( nLen == static_cast<int>(m_papoProperty[i]->GetSrcElementLen()) &&
-            memcmp(pszElement, m_papoProperty[i]->GetSrcElement(), nLen) == 0)
-            return i;
+    auto oIter = m_oMapPropertySrcElementToIndex.find(CPLString(pszElement, nLen));
+    if( oIter != m_oMapPropertySrcElementToIndex.end() )
+        return oIter->second;
 
     return -1;
 }
@@ -189,6 +189,12 @@ int GMLFeatureClass::AddProperty( GMLPropertyDefn *poDefn )
     m_papoProperty[m_nPropertyCount - 1] = poDefn;
     m_oMapPropertyNameToIndex[ CPLString(poDefn->GetName()).toupper() ] =
         m_nPropertyCount - 1;
+    if( m_oMapPropertySrcElementToIndex.find(poDefn->GetSrcElement()) ==
+            m_oMapPropertySrcElementToIndex.end() )
+    {
+        m_oMapPropertySrcElementToIndex[ poDefn->GetSrcElement() ] =
+            m_nPropertyCount - 1;
+    }
 
     return m_nPropertyCount - 1;
 }
@@ -495,8 +501,10 @@ bool GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
                 }
             }
             bHasFoundGeomElements = true;
-            AddGeometryProperty(new GMLGeometryPropertyDefn(
-                pszName, pszElementPath, nGeomType, -1, bNullable));
+            auto poDefn = new GMLGeometryPropertyDefn(
+                pszName, pszElementPath, nGeomType, -1, bNullable);
+            if( AddGeometryProperty(poDefn) < 0 )
+                delete poDefn;
             bHasValidGeometryName = false;
             bHasValidGeometryElementPath = false;
             bHasFoundGeomType = false;
@@ -508,8 +516,10 @@ bool GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
 
             if( bHasValidGeometryName )
             {
-                AddGeometryProperty(new GMLGeometryPropertyDefn(
-                    pszGName, pszGPath, nGeomType, -1, true));
+                auto poDefn = new GMLGeometryPropertyDefn(
+                    pszGName, pszGPath, nGeomType, -1, true);
+                if( AddGeometryProperty(poDefn) < 0 )
+                    delete poDefn;
                 // bHasValidGeometryName = false;
                 bHasValidGeometryElementPath = false;
                 bHasFoundGeomType = false;
@@ -526,8 +536,10 @@ bool GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
 
             if( bHasValidGeometryElementPath )
             {
-                AddGeometryProperty(new GMLGeometryPropertyDefn(
-                    pszGName, pszGPath, nGeomType, -1, true));
+                auto poDefn = new GMLGeometryPropertyDefn(
+                    pszGName, pszGPath, nGeomType, -1, true);
+                if( AddGeometryProperty(poDefn) < 0 )
+                    delete poDefn;
                 bHasValidGeometryName = false;
                 // bHasValidGeometryElementPath = false;
                 bHasFoundGeomType = false;
@@ -544,8 +556,10 @@ bool GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
 
             if( bHasFoundGeomType )
             {
-                AddGeometryProperty(new GMLGeometryPropertyDefn(
-                    pszGName, pszGPath, nGeomType, -1, true));
+                auto poDefn = new GMLGeometryPropertyDefn(
+                    pszGName, pszGPath, nGeomType, -1, true);
+                if( AddGeometryProperty(poDefn) < 0 )
+                    delete poDefn;
                 bHasValidGeometryName = false;
                 bHasValidGeometryElementPath = false;
                 // bHasFoundGeomType = false;
@@ -589,8 +603,10 @@ bool GMLFeatureClass::InitializeFromXML( CPLXMLNode *psRoot )
     if( bHasValidGeometryElementPath || bHasFoundGeomType ||
         !bHasFoundGeomElements )
     {
-        AddGeometryProperty(new GMLGeometryPropertyDefn(pszGName, pszGPath,
-                                                        nGeomType, -1, true));
+        auto poDefn = new GMLGeometryPropertyDefn(pszGName, pszGPath,
+                                                  nGeomType, -1, true);
+        if( AddGeometryProperty(poDefn) < 0 )
+            delete poDefn;
     }
 
     SetSRSName(CPLGetXMLValue(psRoot, "SRSName", nullptr));
@@ -868,6 +884,12 @@ CPLXMLNode *GMLFeatureClass::SerializeToXML()
             CPLCreateXMLElementAndValue(psDSI, "ExtraInfo", m_pszExtraInfo);
     }
 
+    CPLXMLNode* psLastChild = psRoot->psChild;
+    while( psLastChild->psNext )
+    {
+        psLastChild = psLastChild->psNext;
+    }
+
     // Emit property information.
     for( int iProperty = 0; iProperty < GetPropertyCount(); iProperty++ )
     {
@@ -875,7 +897,9 @@ CPLXMLNode *GMLFeatureClass::SerializeToXML()
         const char *pszTypeName = "Unknown";
 
         CPLXMLNode *psPDefnNode =
-            CPLCreateXMLNode(psRoot, CXT_Element, "PropertyDefn");
+            CPLCreateXMLNode(nullptr, CXT_Element, "PropertyDefn");
+        psLastChild->psNext = psPDefnNode;
+        psLastChild = psPDefnNode;
         CPLCreateXMLElementAndValue(psPDefnNode, "Name", poPDefn->GetName());
         CPLCreateXMLElementAndValue(psPDefnNode, "ElementPath",
                                     poPDefn->GetSrcElement());
