@@ -36,7 +36,7 @@
 #include <algorithm>
 #include <set>
 
-CPL_CVSID("$Id: ogrodsdatasource.cpp 59e5ede2cac0762afe7bc68ca19855a5ea42c004 2018-04-21 16:34:01 +0200 Even Rouault $")
+CPL_CVSID("$Id: ogrodsdatasource.cpp a582289d83514f9eb8647ab93d08010a4cb0a080 2019-02-08 14:49:55 +0100 Even Rouault $")
 
 namespace OGRODS {
 
@@ -786,6 +786,63 @@ void OGRODSDataSource::endElementTable( CPL_UNUSED /* in non-DEBUG*/ const char
     }
 }
 
+/************************************************************************/
+/*                           FillRepeatedCells()                        */
+/************************************************************************/
+
+void OGRODSDataSource::FillRepeatedCells(bool wasLastCell)
+{
+    if( wasLastCell && osValue.empty() && osFormula.empty() )
+    {
+        nCellsRepeated = 0;
+        return;
+    }
+
+    if (nCellsRepeated < 0 || nCellsRepeated > 10000)
+    {
+        CPLError(CE_Failure, CPLE_NotSupported,
+                    "Invalid value for number-columns-repeated = %d",
+                    nCellsRepeated);
+        bEndTableParsing = true;
+        nCellsRepeated = 0;
+        return;
+    }
+    const int nFields = nCellsRepeated +
+        (poCurLayer != nullptr ?
+            poCurLayer->GetLayerDefn()->GetFieldCount() : 0);
+    if( nFields > 0 && nRowsRepeated > 100000 / nFields )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                    "Too big gap with previous valid row");
+        bEndTableParsing = true;
+        nCellsRepeated = 0;
+        return;
+    }
+
+    const size_t nCellMemSize =
+        (!osValue.empty()) ? osValue.size() : osFormula.size();
+    if( nCellMemSize > static_cast<size_t>(10 * 1024 * 1024) /
+            (std::max(nCellsRepeated, 1) * nRowsRepeated) )
+    {
+        CPLError(CE_Failure, CPLE_NotSupported,
+                    "Too much memory for row/cell repetition");
+        bEndTableParsing = true;
+        nCellsRepeated = 0;
+        return;
+    }
+    for(int i = 0; i < nCellsRepeated; i++)
+    {
+        if( !osValue.empty() )
+            apoCurLineValues.push_back(osValue);
+        else
+            apoCurLineValues.push_back(osFormula);
+        apoCurLineTypes.push_back(osValueType);
+    }
+
+    nCurCol += nCellsRepeated;
+    nCellsRepeated = 0;
+}
+
 /************************************************************************/
 /*                            startElementRow()                         */
 /************************************************************************/
@@ -793,6 +850,8 @@ void OGRODSDataSource::endElementTable( CPL_UNUSED /* in non-DEBUG*/ const char
 void OGRODSDataSource::startElementRow(const char *pszNameIn,
                                        const char **ppszAttr)
 {
+    FillRepeatedCells(false);
+
     if (strcmp(pszNameIn, "table:table-cell") == 0)
     {
         PushState(STATE_CELL);
@@ -838,38 +897,6 @@ void OGRODSDataSource::startElementRow(const char *pszNameIn,
 
         nCellsRepeated = atoi(
             GetAttributeValue(ppszAttr, "table:number-columns-repeated", "1"));
-        if (nCellsRepeated < 0 || nCellsRepeated > 10000)
-        {
-            CPLError(CE_Failure, CPLE_NotSupported,
-                     "Invalid value for number-columns-repeated = %d",
-                     nCellsRepeated);
-            bEndTableParsing = true;
-            nCellsRepeated = 0;
-            return;
-        }
-        const int nFields = nCellsRepeated +
-            (poCurLayer != nullptr ?
-                poCurLayer->GetLayerDefn()->GetFieldCount() : 0);
-        if( nFields > 0 && nRowsRepeated > 100000 / nFields )
-        {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                     "Too big gap with previous valid row");
-            bEndTableParsing = true;
-            nCellsRepeated = 0;
-            return;
-        }
-
-        const size_t nCellMemSize =
-            (!osValue.empty()) ? osValue.size() : osFormula.size();
-        if( nCellMemSize > static_cast<size_t>(10 * 1024 * 1024) /
-                (std::max(nCellsRepeated, 1) * nRowsRepeated) )
-        {
-            CPLError(CE_Failure, CPLE_NotSupported,
-                     "Too much memory for row/cell repetition");
-            bEndTableParsing = true;
-            nCellsRepeated = 0;
-            return;
-        }
     }
     else if (strcmp(pszNameIn, "table:covered-table-cell") == 0)
     {
@@ -891,6 +918,8 @@ void OGRODSDataSource::endElementRow( CPL_UNUSED /*in non-DEBUG*/ const char * p
     {
         CPLAssert(strcmp(pszNameIn, "table:table-row") == 0);
 
+        FillRepeatedCells(true);
+
         /* Remove blank columns at the right to defer type evaluation */
         /* until necessary */
         size_t i = apoCurLineTypes.size();
@@ -1072,6 +1101,7 @@ void OGRODSDataSource::endElementRow( CPL_UNUSED /*in non-DEBUG*/ const char * p
                             OGRFieldDefn oNewFieldDefn(
                                 poCurLayer->GetLayerDefn()->GetFieldDefn(
                                     static_cast<int>(i)));
+                            oNewFieldDefn.SetSubType(OFSTNone);
                             if( ( eFieldType == OFTDate ||
                                   eFieldType == OFTTime) &&
                                 eValType == OFTDateTime )
@@ -1140,17 +1170,6 @@ void OGRODSDataSource::endElementCell( CPL_UNUSED /*in non-DEBUG*/ const char *
     if (stateStack[nStackDepth].nBeginDepth == nDepth)
     {
         CPLAssert(strcmp(pszNameIn, "table:table-cell") == 0);
-
-        for(int i = 0; i < nCellsRepeated; i++)
-        {
-            if( !osValue.empty() )
-                apoCurLineValues.push_back(osValue);
-            else
-                apoCurLineValues.push_back(osFormula);
-            apoCurLineTypes.push_back(osValueType);
-        }
-
-        nCurCol += nCellsRepeated;
     }
 }
 
@@ -1843,6 +1862,11 @@ void OGRODSDataSource::FlushCache()
 
     osTmpFilename = CPLSPrintf("/vsizip/%s/META-INF/manifest.xml", pszName);
     VSILFILE* fp = VSIFOpenL(osTmpFilename, "wb");
+    if( fp == nullptr )
+    {
+        VSIFCloseL(fpZIP);
+        return;
+    }
     VSIFPrintfL(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
     VSIFPrintfL(
         fp,
@@ -1866,6 +1890,11 @@ void OGRODSDataSource::FlushCache()
 
     osTmpFilename = CPLSPrintf("/vsizip/%s/meta.xml", pszName);
     fp = VSIFOpenL(osTmpFilename, "wb");
+    if( fp == nullptr )
+    {
+        VSIFCloseL(fpZIP);
+        return;
+    }
     VSIFPrintfL(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
     VSIFPrintfL(
         fp, "<office:document-meta "
@@ -1876,6 +1905,11 @@ void OGRODSDataSource::FlushCache()
 
     osTmpFilename = CPLSPrintf("/vsizip/%s/settings.xml", pszName);
     fp = VSIFOpenL(osTmpFilename, "wb");
+    if( fp == nullptr )
+    {
+        VSIFCloseL(fpZIP);
+        return;
+    }
     VSIFPrintfL(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
     VSIFPrintfL(
          fp, "<office:document-settings "
@@ -1932,6 +1966,11 @@ void OGRODSDataSource::FlushCache()
 
     osTmpFilename = CPLSPrintf("/vsizip/%s/styles.xml", pszName);
     fp = VSIFOpenL(osTmpFilename, "wb");
+    if( fp == nullptr )
+    {
+        VSIFCloseL(fpZIP);
+        return;
+    }
     VSIFPrintfL(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
     VSIFPrintfL(
          fp, "<office:document-styles "
@@ -1948,6 +1987,11 @@ void OGRODSDataSource::FlushCache()
 
     osTmpFilename = CPLSPrintf("/vsizip/%s/content.xml", pszName);
     fp = VSIFOpenL(osTmpFilename, "wb");
+    if( fp == nullptr )
+    {
+        VSIFCloseL(fpZIP);
+        return;
+    }
     VSIFPrintfL(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
     VSIFPrintfL(
          fp, "<office:document-content "
